package system

import (
	"fmt"

	gnmitargetygot "code.fbi.h-da.de/danet/gnmi-target/examples/example01/model"
	"code.fbi.h-da.de/danet/gnmi-target/examples/example01/osclient"
	"code.fbi.h-da.de/danet/gnmi-target/handler"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
	"github.com/sirupsen/logrus"
)

// HostnameHandler is the implementation of a gnmitarget.PathHandler.
type HostnameHandler struct {
	handler.DefaultPathHandler
	osClient osclient.Osclient
	weight   int
}

func NewHostnameHandler() *HostnameHandler {
	return &HostnameHandler{
		DefaultPathHandler: handler.DefaultPathHandler{
			Name: "openconfig-hostname-handler",
			Paths: map[string]struct{}{
				"/system/config/hostname": {},
			},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *HostnameHandler) Init(config *handler.Config, publishToSubsFunc func([]*gnmi.Notification) error) error {
	yh.Config = config
	yh.PublishToSubs = publishToSubsFunc

	_, err := yh.updateOrCreateHostname(yh.osClient)
	if err != nil {
		return err
	}
	return nil
}

func (yh *HostnameHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.Name)
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	system := config.GetSystem()

	if system != nil {
		if config := system.GetOrCreateConfig(); config != nil {
			if hostname := config.GetHostname(); hostname != "" {
				err := yh.osClient.SetHostname(&hostname)
				if err != nil {
					logrus.Debug("Failed to set hostname: ", err)
					return err
				}
			}
		}
	}
	return nil
}

func (yh *HostnameHandler) updateOrCreateHostname(os osclient.Osclient) ([]*gnmi.Notification, error) {
	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return nil, err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return nil, fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	confSystem := newConfig.GetOrCreateSystem()

	if config := confSystem.GetOrCreateConfig(); config != nil {
		h, err := os.GetHostname()
		if err != nil {
			return nil, err
		}
		config.Hostname = &h
	}

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return nil, err
	}

	notifications, err := ygot.DiffWithAtomic(yh.Config.Data, newConfig)
	if err != nil {
		return nil, err
	}

	yh.Config.Data = newConfig

	return notifications, nil
}
